function [Mapping DRdata] = DRsublib(obj,prms)
%% This is small lib which contains five techniques for compacting high dimensional vectors 
%%  with dimensionality reduction: 
%%1. unsupervised: PCA, Whitening PCA, Random Projection
%%2. supervised: LDA
%%3. learning-based: Low-rank metric learning
%%For stablely runing LDA, and Low-rank metric learning,
%%three unsupervised methods will be as an initialization. 
%%This is written by Qilong Wang (Email:  qlwang at mail dot dlut dot edu dot cn)

switch prms.type
    case 'PCA'
        %%Linear PCA transformation
        
        Lpca.dps =  prms.dimprops;
        Lpca.dp =  prms.dimprop;
        Lpca.dim = prms.dim;
        Lpca.do_whitening = false; 
        [Mapping, DRdata] = LinPCA(obj.trainX, Lpca);
        Mapping.proj = real(Mapping.proj);
        Mapping.mu = real(Mapping.mu);
    case 'WPCA'    
        %%Linear whitening PCA transformation
        Lpca.dps =  prms.dimprops;
        Lpca.dp =  prms.dimprop;
        Lpca.dim = prms.dim;
        Lpca.do_whitening = true; 
        [Mapping, DRdata] = LinPCA(obj.trainX, Lpca);
        
    case 'RP'  
        %%Random Projection
        if prms.dimprops
            Ldim = round(size(obj.trainX,1)*prms.dimprop);
        else
            Ldim = prms.dim;
        end
        R = randn(Ldim, size(obj.trainX,1));
        R_norm = sqrt(sum(R.*R, 2));
        Mapping.proj = R./repmat(R_norm, 1, size(R, 2));
        DRdata = Mapping.proj*obj.trainX;
        
    case {'LDA','RSR','LRSVM'} 

        %% Initialization with 'PCA', 'WPCA', 'RP'
        if strcmp(prms.init,'PCA')==1
            
                Lpca.dim = prms.initdim;
                Lpca.do_whitening = false; 
                [~, Initdata] = LinPCA(obj.trainX, Lpca);
                
        elseif strcmp(prms.init,'WPCA')==1
            
                Lpca.dim = prms.initdim;
                Lpca.do_whitening = true; 
                [~, Initdata] = LinPCA(obj.trainX, Lpca);
                
        elseif strcmp(prms.init,'RP')==1
            
               R = randn(prms.initdim,prms.data_dim);
               R_norm = sqrt(sum(R.*R, 2));
               Mapping.proj = R./repmat(R_norm, 1, size(R, 2));
               Initdata = Mapping.proj*obj.trainX;
               
        else
%             error('This kind of initialization does not exist! \n');
            warning('This kind of initialization does not exist!  It maybe make an error due to high dimensional vectors!');
            Initdata = obj.trainX;
        end
        
        if strcmp(prms.type,'LDA')==1
        %%Linear discriminant analysis  LDA
            [DRdata, mapping] = lda(Initdata, obj.labels, prms.dim);
            Mapping.proj =  mapping.M;
            Mapping.mu =  mapping.mean;  
            
        elseif strcmp(prms.type,'RSR')==1
        %%Rotated Sparse Regression
            error('Not yet!');
        elseif strcmp(prms.type,'LRSVM')==1
            error('Not yet!');
        end
             
end