function lr_basis = DimensionRed(traindata,trainlabel,featextr)

% DimensionRed learns low rank matrix for dimension reduction 
%
% For  theoretical and technical details, please refer to the following paper:
%
% Qilong Wang, Peihua Li, Wangmeng Zuo, and Lei Zhang. Towards Effective
% Codebookless Model for Image Classification. Pattern Recognition, 2016.
% DOI: 10.1016/j.patcog.2016.03.004 
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Qilong Wang  (Email:  qlwang at mail dot dlut dot edu dot cn), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

prms = setprms(featextr);
obj.trainX = traindata;
obj.labels = trainlabel;

fprintf('Reducing dimension of each CLM to %d with performing %s ...\n',prms.dim,prms.type);

if featextr.repartion
    
   if strcmp(prms.type,'LRSVM') == 1
       
    prms.type = 'PCA';
    
    spm = featextr.spm(:).^2;
    allnum = sum(spm);
    lr_basis = cell(allnum,1);
    dprerspm = size(traindata,1)/allnum;
    
    for ii = 1:allnum
        obj.trainX = traindata((ii-1)*dprerspm+1:ii*dprerspm,:);
        [lr_basisper, ~] = DRsublib(obj,prms);   
        lr_basis{ii} = lr_basisper;  
    end
    
   trdata = [];
    for ii = 1:allnum
          trdataT = traindata((ii-1)*dprerspm+1:ii*dprerspm,:);
          trdataT =  bsxfun(@minus, trdataT, lr_basis{ii}.mu);
          trdata = [trdata; trdataT];
    end
   
   [Classifier,~]=lrsvm(trdata,trainlabel,lr_basis,featextr.maxiters);
        
    for ii = 1:allnum
        lr_basis{ii}.proj = Classifier.L{ii};
    end
   
    
   else
       
    spm = featextr.spm(:).^2;
    allnum = sum(spm);
    lr_basis = cell(allnum,1);
    dprerspm = size(traindata,1)/allnum;
    
    for ii = 1:allnum
        obj.trainX = traindata((ii-1)*dprerspm+1:ii*dprerspm,:);
        [lr_basisper, ~] = DRsublib(obj,prms);   
        lr_basis{ii} = lr_basisper;  
    end  
       
   end    
else

    if strcmp(prms.type,'LRSVM') == 1
        prms.type = 'PCA';
        [lr_basis, ~] = DRsublib(obj,prms); 
        traindata =  bsxfun(@minus, traindata, lr_basis.mu);
        [Classifier,~]=lrsvm(traindata,trainlabel,lr_basis);
        lr_basis.proj = Classifier.L;
    else
        [lr_basis, ~] = DRsublib(obj,prms); 
    end
    
end

end