function [Classifier,E]=lrsvm(X,Label,initL,maxit,tol,svmC)

% lrsvm learns low rank matrix joint with SVM for dimension reduction 
%
% For  theoretical and technical details, please refer to the following paper:
%
% Qilong Wang, Peihua Li, Wangmeng Zuo, and Lei Zhang. Towards Effective
% Codebookless Model for Image Classification. Pattern Recognition, 2016.
% DOI: 10.1016/j.patcog.2016.03.004 
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Qilong Wang  (Email:  qlwang at mail dot dlut dot edu dot cn), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.


% ---------- Argument defaults ----------
if ~exist('tol','var') || isempty(tol)
  tol=1e-5;
end
if ~exist('maxit','var') || isempty(maxit)
  maxit=1; %1000
end
if ~exist('svmC','var') || isempty(maxit)
  svmC=10;
end

% ---------- End of "argument defaults" ----------

% Number of One-Versus-All SVM classifier.
Y =unique(Label);
ClassN = length(Y);
% Fit mappings L and SVM.
i=0;   
libsvm=cell(1,ClassN);  

svm_options=['-q -t 4 -c ' num2str(svmC)];
Ldim = size(initL{1}.proj,1);
L = cell(length(initL),1);

for kk = 1:length(initL)
    L{kk} = real(initL{kk}.proj);
end
spm = size(X,1)/length(initL);
E = [];

while i<maxit
  i=i+1; 
  
  %% Compute Z with L and X
        Z = [];
        for kk = 1:length(initL)
           Xt = X((kk-1)*spm+1:kk*spm,:);
           Zt = L{kk}*Xt;
           Z = [Z; Zt];
        end
        Z = Z';
        Z = double(Z);
KZ = Z*Z';
  %% Optimization over w and xi using  One-Versus-All SVM.
%   parfor j=1:ClassN  
  for j=1:ClassN  
      Yj=-ones(size(Label,1),1); 
      Yj(Label==j) = 1;
      KernelMartixTrain = [(1:size(KZ,1))',KZ];
      g = svmtrain(Yj, KernelMartixTrain, svm_options);  
      libsvm{j}=g;
  end
  
%   if i>1&&abs(E(end)-E(end-1)) < tol
%       break;
%   end
  
  %%  Optimization over L with fixed w and xi.
  S = zeros(size(X,2));
  %   parfor j=1:ClassN  

WLM = cell(ClassN,1);
  for j=1:ClassN  
      Yj=-ones(size(Label,1),1); 
      Yj(Label==j) = 1;    

      g = libsvm{j};
      wpos = full(g.SVs);
      fullwj = zeros(size(Z,1),1);
      fullwj(wpos) = abs(g.sv_coef);

      WLM{j} = diag(Yj)*fullwj;
      H = fullwj*fullwj';
      S = S + diag(Yj)*H*diag(Yj); 
  end  
 
 [US,SS,~] = svd(S);
 diag_SS = diag(SS);
 diag_SS2 = abs(diag_SS).^(0.5);
 S2 = US*diag(diag_SS2)*US';
 EERS = S-S2*S2;
 
 LEER = 0;
    
 for kk = 1:length(initL)
      Xt = X((kk-1)*spm+1:kk*spm,:);

     %% 
      Mt = Xt*S2;
      Mt = double(Mt);
      [Eigvec, ~, ~] = princomp(Mt', 'econ');
      Ut = Eigvec(:, 1:Ldim);
      
      Ut = Ut';
      LEER = LEER + norm(L{kk}-Ut,'fro'); 
      L{kk} = Ut;

 end

 LEER = LEER/length(initL);
 E = [E LEER];

end
Classifier.L = L;
Classifier.svm = libsvm;
fprintf('Done.\n');
