%% 
% Run_demo_fmd--Demo code to evaluate the proposed codebookless model (CLM) using 
% the Flickr material database.
% For  theoretical and technical details, please refer to the following paper:
%
% Qilong Wang, Peihua Li, Wangmeng Zuo, and Lei Zhang. Towards Effective
% Codebookless Model for Image Classification. Pattern Recognition, 2016.
% DOI: 10.1016/j.patcog.2016.03.004 
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Qilong Wang  (Email:  qlwang at mail dot dlut dot edu dot cn), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

clc;
root = pwd;
addpath(fullfile(root, 'feaextra'));
addpath(fullfile(root, 'l2emg'));
addpath(fullfile(root, 'common'));
addpath(fullfile(root, 'DimRed'));
addpath(fullfile(root, 'classifier'));
addpath(fullfile(root, 'vlfeat-0.9.18\toolbox'));
run vl_setup;
% if matlabpool('size') == 0
%      n = feature('numCores');
%     matlabpool('local', n);
% end

% exps ={{'sift',0.65,'NONE',1e5},{'esift',1.0,'NONE',1e5},{'L2EMG',0.75,'NONE',1e5},{'eL2EMG',0.65,'NONE',1e5},{'esift',1.0,'LRSVM',450},{'esift',1.0,'PCA',64},{'esift',1.0,'LRSVM',64}};

exps ={{'esift',1.0,'NONE',1e5},{'esift',1.0,'LRSVM',450},{'esift',1.0,'PCA',64},{'esift',1.0,'LRSVM',64}};

allacc = zeros(length(exps),2);

for k = 1:length(exps)
        
s = RandStream('mt19937ar','Seed',1436);
RandStream.setGlobalStream(s);

total_start_time = round(clock);

expopt = exps{k};

exprs = plow_expriments('10',expopt{1,1});
feat_prms = plow_features(expopt);

%% Extract  features of every image per category
fprintf('****************************************************************\n');
fprintf('Extract features of every image per category\n');
fprintf('****************************************************************\n');
start_time = round(clock);

categories_dir0 = dir(exprs.base_image_dir);
categories_dir = struct([]);
for i=1:length(categories_dir0)
    if strcmp(categories_dir0(i).name, '.') == 1 ||  strcmp(categories_dir0(i).name, '..') == 1 || categories_dir0(i).isdir == 0
        continue;
    end
    categories_dir = [categories_dir; categories_dir0(i)];
end

featperclass = cell(length(categories_dir),1); 

% parfor i=1: length(categories_dir)
for i=1: length(categories_dir)
   
    image_dir = fullfile(exprs.base_image_dir, sprintf('%s', categories_dir(i).name));
    data_dir   =  fullfile(exprs.base_feature_dir,  exprs.image_dataset,   sprintf('%s', categories_dir(i).name));
    featperclass{i} = FeatureExtraction(image_dir,data_dir,feat_prms); 
    
end
end_time = round(clock);
elapsed_time = end_time - start_time;
fprintf('Time taken: %5d days %5d hours %5d minutes %5d seconds\n\n', elapsed_time(3), elapsed_time(4), elapsed_time(5), elapsed_time(6));


%% Split training set and test set, and compute  kernel matrix and peform SVM training
fprintf('****************************************************************\n');
fprintf('Peform SVM training and test\n');
fprintf('****************************************************************\n');

iter_max = 5;
start_time = round(clock);
accuracies = zeros(iter_max, 1);

for iter = 1: iter_max
    
     [trainlist,testlist] = random_split_training_test(exprs,categories_dir);
     [traindata, testdata, trainlabel, testlabel] =  get_data_label(categories_dir,featperclass,trainlist,testlist);
     
     %%%%%%% Dimension Reduction
    
     lr_basis = [];
     if strcmp(feat_prms.reduction,'NONE')~=1
         lr_basis = DimensionRed(traindata,trainlabel,feat_prms);
     end
     
     if ~isempty(lr_basis) 
         [traindata, testdata] = processingdata(traindata,testdata,lr_basis);
     end
 
     accuracy = SVMforClassification(categories_dir,traindata,testdata,trainlabel,testlabel,exprs);
     accuracies(iter) = accuracy;

end

allacc(k,1) =  mean(accuracies)*100;
allacc(k,2) =  std(accuracies)*100;

fprintf('Average accuracy of CLM with %s and %s dim. redu. over %d times: mean+std %f+%f\n',expopt{1,1} , expopt{1,3} ,iter_max, mean(accuracies)*100, std(accuracies)*100);
end_time = round(clock);
elapsed_time = end_time - start_time;
fprintf('Time taken: %5d days %5d hours %5d minutes %5d seconds\n\n', elapsed_time(3), elapsed_time(4), elapsed_time(5), elapsed_time(6));
total_end_time = round(clock);
fprintf('Total time of the classification program is:\n');
   
disp(total_start_time);
disp(total_end_time);

end

res_summary(exps,allacc); 

%matlabpool close ;