function  features = FeatureExtraction(image_dir,data_dir,feat_prms) 

% FeatureExtraction computes the CLM for all images
%
% Qilong Wang, Peihua Li, Wangmeng Zuo, and Lei Zhang. Towards Effective
% Codebookless Model for Image Classification. Pattern Recognition, 2016.
% DOI: 10.1016/j.patcog.2016.03.004 
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Qilong Wang  (Email:  qlwang at mail dot dlut dot edu dot cn), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

fnames = dir(fullfile(image_dir, '*.jpg'));
imageFileList = cell( length(fnames),1);
for f = 1: length(fnames)
    imageFileList{f} = fnames(f).name;
end

imageFName = fullfile(image_dir, imageFileList{1});
I = imread(imageFName);
I = imresize(I, [16 16], 'bicubic'); 
[RawFeas, ~] = GenerateRawFea(I,feat_prms);
if strcmp(feat_prms.feature_type,'gaussian')==1
    ddim = size(RawFeas,1)+1;
    single_desc_dim = ddim*(ddim+1)/2;
else
    single_desc_dim = (size(RawFeas,1)*(size(RawFeas,1)+1)/2);
end

feat_dim = sum((feat_prms.spm).^2) * single_desc_dim;
features = zeros(feat_dim, numel(imageFileList), 'single');
shift = feat_prms.constrant.*eye(size(RawFeas,1));
true_mat = true(size(RawFeas,1),size(RawFeas,1));
in_triu = triu(true_mat);
fprintf('Extracting %s features--Processing %s', feat_prms.feature_type,image_dir);

for f =1:size(imageFileList,1)

    if ~mod(f, 5)
        fprintf('.');
    end
    
    curr_filename = imageFileList{f};

    suffix_pos = findstr(curr_filename, '.jpg');
    outFName = fullfile(data_dir, [curr_filename(1:suffix_pos-1) '.mat']);
    if feat_prms.canSkip && size(dir(outFName), 1)
        load(outFName);
        features(:,f) = Fea_each_im;
        continue;
    end
    
    imageFName = fullfile(image_dir, curr_filename);
    I = imread(imageFName);
    
    [hgt, wid, ~] = size(I);
    if max(hgt,wid) > feat_prms.maxImageSize    
        I = imresize(I, feat_prms.maxImageSize/max(hgt,wid), 'bicubic'); 
    end
   
    [RawFeas, RawPos] = GenerateRawFea(I,feat_prms);
    Fea_each_im = IndividualModelIm(I,RawFeas,RawPos,feat_prms,single_desc_dim,in_triu,shift);   
    Fea_each_im = squash_features(Fea_each_im,feat_prms.norm);
  
    Fea_each_im = single(Fea_each_im);
    features(:,f) = Fea_each_im;
    sp_make_dir(outFName);
    save(outFName, 'Fea_each_im');   
    
end
fprintf('Completed.\n');

