function  Fea_each_im = IndividualModelIm(I,D,F,feat_prms,single_desc_dim,in_triu,shift)

% IndividualModelIm represents each image by using codebookless model 
%
% For  theoretical and technical details, please refer to the following paper:
%
% Qilong Wang, Peihua Li, Wangmeng Zuo, and Lei Zhang. Towards Effective
% Codebookless Model for Image Classification. Pattern Recognition, 2016.
% DOI: 10.1016/j.patcog.2016.03.004 
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Qilong Wang  (Email:  qlwang at mail dot dlut dot edu dot cn), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

y = F(1,:);
x = F(2,:);
y = y - min(y) + 1 ;
x = x - min(x) + 1;

pgrid = feat_prms.spm.^2;
Fea_each_im = zeros(sum(pgrid)*single_desc_dim,1);
% pool local sift descriptors over a spatial pyramid

weights = (1./feat_prms.spm); % divided by the number of grids at the corresponding level
weights = weights/sum(weights);

counter = 1;
for s = 1:length(feat_prms.spm)
       width = (size(I,2)-1)/feat_prms.spm(s);
       height = (size(I,1)-1)/feat_prms.spm(s);
       xgrid = ceil(x/height);
       ygrid = ceil(y/width);
       allgrid = (ygrid -1 )*feat_prms.spm(s) + xgrid;
       for t = 1:pgrid(s)
           range = counter:counter+single_desc_dim-1;
           ind = find(allgrid == t);
           if(numel(ind)>1)
               theD = D(:,ind);
               if strcmp(feat_prms.feature_type,'mean')==1
                    regionD = mean(theD);
               elseif strcmp(feat_prms.feature_type,'cov_mean')==1
         
                   regionD = real(logm(((1/size(theD,2)).*(theD *theD')) + shift)); 
                   regionD = regionD(in_triu);              
                   
               elseif strcmp(feat_prms.feature_type,'cov')==1
                   
                   DataD = cov(theD') + shift;
                   [U S V] = svd(DataD);
                   diag_S = diag(S);
                   diag_S = log(diag_S);  
                   regionD = U*diag(diag_S)*U';
                   regionD = regionD(in_triu); 
                   
               elseif strcmp(feat_prms.feature_type,'gaussian')==1
                   true_mat = true(size(D,1)+1,size(D,1)+1); 
                   in_triu = triu(true_mat);
                   regionD = ones(size(theD,1)+1); 
                 
                   COVD = cov(theD');
                   [U1, S1] = eig(COVD);             
                   diag_S1 = diag(S1) + feat_prms.constrantGaussian;
                   diag_S1 = sign(diag_S1).*abs(diag_S1).^feat_prms.embedding_pn;
                   COVD = U1*(diag(diag_S1))*U1'; 
                       
                   b = feat_prms.embedding_beta;
                   b2 = b^2;
                   regionD(1:size(theD,1),1:size(theD,1)) = (COVD + shift + b2.*(mean(theD,2)*mean(theD,2)')); 
                   regionD(1:size(theD,1),1+size(theD,1)) =  b.*mean(theD,2);
                   regionD(1+size(theD,1),1:size(theD,1)) =  b.*mean(theD,2)';
                      
                   [U, S] = eig(regionD);             
                   diag_S = diag(S);                  
                   diag_S = diag_S + feat_prms.constrantGaussian;
                   log_diag_S = log(diag_S);
                   regionD = U*(diag(log_diag_S))*U'; 
                   regionD = regionD(in_triu);
               end
               Fea_each_im(range) = weights(s) * regionD(:);
           end
           counter = counter + single_desc_dim;
       end
end
end