function [fimage] = buildfeaturesRGB(inimage)
% buildfeatures generates raw feature for L2ECM image feature.
%
% Please cite the following paper if you use the code:
%
%Peihua Li,  Qilong Wang: Local Log-Euclidean Covariance Matrix (L2ECM) for Image 
%Representation and Its Applications, (ECCV) 2012.
% For questions,  please conact:  Qilong Wang  (Email:  wangqilong.415@163.com), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

[height width depth] = size(inimage);

fdim = 22;
fimage = zeros(height, width, fdim);

%%%%%%%%%%%% Color information
fimage(:,:,1)   = im2double(inimage(:,:,1));
fimage(:,:,2)   = im2double(inimage(:,:,2));
fimage(:,:,3)   = im2double(inimage(:,:,3));

%%%%%%%%%%% Location
[x y]=meshgrid(1:width, 1:height);
x = x / width; 
y = y / height; 

fimage(:,:,4)   = x;
fimage(:,:,5)   = y;

% gimage = im2double(rgb2gray(inimage));
gimage = double(rgb2gray(inimage));
SIGMA = 2;
gimage = vl_imsmooth(gimage, SIGMA);

%%%%%%%%%%% harris
harris = vl_harris(gimage , 2 , [] );
fimage(:,:,6)   = harris;

%%%%%%%%%%% Gradient
filter = [-3 5 5;-3 0 5;-3 -3 -3] / 15; 
filter1 = [5 5 -3;5 0 -3;-3 -3 -3] / 15; 
himage = conv2(gimage, filter, 'same');
vimage = conv2(gimage, filter1, 'same');
fimage(:,:,7)   = abs(himage);
fimage(:,:,8)   = abs(vimage);
shimage = conv2(himage, filter, 'same');
svimage = conv2(vimage, filter1, 'same');
fimage(:,:,9)   = abs(shimage);
fimage(:,:,10)   = abs(svimage);


s2 = sqrt(2);
FreiChen_filter1= [1 0 -1; s2 0 -s2; 1 0 -1]/(2+s2);
FreiChen_filter2=[-1 -s2 -1; 0 0 0; 1 s2 1]/(2+s2);
himage = conv2(gimage, FreiChen_filter1, 'same');
vimage = conv2(gimage, FreiChen_filter2, 'same');
fimage(:,:,11)   = abs(himage);
fimage(:,:,12)    = abs(vimage);
shimage = conv2(himage, FreiChen_filter1, 'same');
svimage = conv2(vimage, FreiChen_filter2, 'same');
fimage(:,:,13)   = abs(shimage);
fimage(:,:,14)   = abs(svimage);

%%%%%%%%%%% Gabor features
Gabor_options.Width=17;
Gabor_options.num_theta=4;
Gabor_options.num_scale=2;
[GR,GI]=Create_GaborFilters (Gabor_options);
Gabor_F = Apply_Gabor_Transform(gimage,1,GR, GI);
fimage(:,:,15:22)   = Gabor_F;

%fimage = bsxfun(@rdivide, fimage, sqrt(sum(sum((fimage .^2)))));


end

