function [ feature ] = covcalculatorwithcheck(inimage, patch_size, sampling_step, step)
% covcalculatorwithcheck(detwindow, patch_size, sampling_step, step) compute L2ECM feature image
%
% Input:
% inimage: input images
% patch_size: the size of a patch on which to compute the descriptor
% sampling_step:  The sampling spacing of dense descriptors
%
% Ouput:
% feature: L2ECM features
%
% Please cite the following paper if you use the code:
%
%Peihua Li,  Qilong Wang: Local Log-Euclidean Covariance Matrix (L2ECM) for Image 
%Representation and Its Applications, (ECCV) 2012.
%
% For questions,  please conact:  Qilong Wang  (Email:  wangqilong.415@163.com), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

%% Compute raw features
im_features = buildfeatures(inimage); 

%% Compute dense L2ECM features
n=size(im_features,3);
ones_mat=ones(n,n);
ones_mat = triu(ones_mat, 1) * sqrt(2) + eye(n);
triu_idx=find(ones_mat>0);
ones_vec = ones_mat(triu_idx);

[im_covs im_mask grid_x grid_y]  = dense_cov_integral(im_features, patch_size, sampling_step);   
%local_covs = zeros(size(im_covs,1),size(im_covs,2),length(triu_idx));
local_covs =im_covs(:,:,triu_idx);

local_covs2 = shiftdim(local_covs, 2);
local_covs2 = bsxfun(@times, local_covs2, ones_vec);
local_covs = shiftdim(local_covs2, 1);

    
%% check local log cov descriptors
[my mx] = size(im_mask);   
[y x]= find( im_mask < n); 
len = length(x);
[use_x use_y] = find(im_mask == n,1,'first');
use_l = length(use_x);
if use_l < 1
   error('All local log-Euclidean covariance matrix descriptors are  ill conditioned !');
end    
bound = 2;
if len >= 1
    for k =1:len
        lx = max(x(k)-bound,1);    
        rx = min(x(k)+bound,mx);
        ly = max(y(k)-bound,1);    
        ry = min(y(k)+bound,my);
        [use_idx use_idy] = find(im_mask(ly:ry,lx:rx) == n,1,'first');
        local_covs_t = local_covs(ly:ry,lx:rx,:);
        use_len = length(use_idx);
        if use_len >= 1
          local_covs(y(k),x(k),:) =  local_covs_t(use_idx,use_idy,:); 
        else
          local_covs(y(k),x(k),:) =  local_covs(use_x,use_y,:);   
        end
    end
end

feature = local_covs;

end