function [sift_cov,sift_cov_mask, grid_x, grid_y] = dense_cov_integral(I_orientation, patch_size, grid_spacing)

sample_step = grid_spacing;
grid_spacing = 1;

[hgt wid num_angles] = size(I_orientation);
n = num_angles;
pixels_num = patch_size * patch_size;
inte_features = integralfeatures(I_orientation);



grid_x = patch_size/2:grid_spacing:wid-patch_size/2;
grid_y = patch_size/2:grid_spacing:hgt-patch_size/2;
[sx1 sx] = size(grid_x);
[sy1 sy] = size(grid_y);
% Sample SIFT bins at valid locations (without boundary artifacts)
% find coordinates of sample points (bin centers)


[Hgt Wid layer] = size(inte_features);
window_sum = zeros(sy, sx, layer);

for i = 1 : layer
window_sum(:,:,i) = inte_features(patch_size+1 : end, patch_size+1 : end, i) + inte_features(1 : end-patch_size, 1 : end-patch_size, i) ...
                  - inte_features(patch_size+1 : end, 1 : end-patch_size, i) - inte_features(1 : end-patch_size, patch_size+1 : end, i);
end

cov_vector1 = zeros(sy, sx, n, n);
i_b = n; num = 0;
for i = 1 : n
    i_b = i_b + num;
    layer_idx = i_b + i;
    num = 1;
    cov_vector1(:,:,i,i) = window_sum(:,:,layer_idx) / (pixels_num-1) - window_sum(:,:,i) .* window_sum(:,:,i)  / ((pixels_num-1)* pixels_num);
    for j = i+1 : n
        layer_idx = i_b + j;
        cov_vector1(:,:,i,j) = window_sum(:,:,layer_idx) / (pixels_num-1) - window_sum(:,:,i) .* window_sum(:,:,j)  / ((pixels_num-1)* pixels_num);
        cov_vector1(:,:,j,i) = cov_vector1(:,:,i,j);
        num = num + 1;
    end
    num = num -1;
end

data_cov       = zeros(num_angles, num_angles);
sift_cov_mask  = zeros([length(grid_y) length(grid_x)],'uint8');
U=zeros(num_angles,num_angles);
S=zeros(num_angles,num_angles);
V=zeros(num_angles,num_angles);
diag_S=zeros(num_angles,1);

for y=1:sy
    for x=1:sx
      
        data_cov = reshape(cov_vector1(y,x,:,:), n, n);

        [U S V]=svd(data_cov);
        diag_S=diag(S);
        idx = find(diag_S>num_angles*eps(diag_S(1,1)));
        sift_cov_mask(y,x) = length(idx);
        data_cov = U*diag(log(diag_S))*V';
        
        sift_cov(y,x,:)=reshape(data_cov, size(data_cov,1)*size(data_cov,2),1);
    end
end
%oneD_sift_cov = reshape(sift_cov,size(sift_cov,1)*size(sift_cov,2), size(sift_cov,3));

data_cov       = [];

if sample_step>1
    [h_s w_s d_s]= size(sift_cov);
    sift_cov = sift_cov(1:sample_step:h_s,1:sample_step:w_s,:);
    sift_cov_mask = sift_cov_mask(1:sample_step:h_s,1:sample_step:w_s);
    grid_x = grid_x(1:sample_step:w_s);
    grid_y = grid_y(1:sample_step:h_s);
end

% Outputs:
%[grid_x,grid_y] = meshgrid(grid_x, grid_y);

